'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var fs = require('fs');
var path = require('path');

var minimist = require('minimist');
var find = require('fs-find-root');
var bole = require('bole');
var garnish = require('garnish');

var createFrockInstance = require('./');
var createWatcher = require('./watcher');

module.exports = processCli;

function processCli(args, _process, ready) {
  var pc = _process || process;

  if (typeof pc === 'function') {
    pc = process;
    ready = _process;
  }

  var options = {
    alias: {
      nowatch: 'w',
      debug: 'd',
      raw: 'r'
    },
    boolean: ['nowatch', 'debug', 'raw', 'unsafe-disable-connection-filtering'],
    default: {
      nowatch: Boolean(pc.env.FROCK_NO_WATCH),
      debug: Boolean(pc.env.FROCK_DEBUG),
      raw: Boolean(pc.env.FROCK_RAW_OUTPUT),
      'unsafe-disable-connection-filtering': Boolean(pc.env.FROCK_UNSAFE_DISABLE_CONNECTION_FILTERING)
    }
  };
  var argv = minimist(args, options);
  var logLevel = argv.debug ? 'debug' : 'info';

  var logOutput = pc.stdout;

  if (!argv.raw) {
    logOutput = garnish({ level: logLevel });
    logOutput.pipe(pc.stdout);
  }

  bole.output({
    level: logLevel,
    stream: logOutput
  });

  if (argv.version || argv.help) {
    return ready(null, argv);
  }

  var file = argv._[0];

  // if we weren't given a frockfile path, find one
  if (!file) {
    find.file('frockfile.json', pc.cwd(), function (err, foundFile) {
      if (err) {
        return ready(err);
      }

      file = foundFile;

      fs.readFile(file, onFrockfile);
    });

    return;
  }

  // otherwise, we had a path, so just read the frockfile
  fs.readFile(file, onFrockfile);

  function onFrockfile(err, _frockfile) {
    if (err) {
      return ready(err);
    }

    argv.pwd = path.dirname(file);

    var frockfile = void 0;

    try {
      frockfile = JSON.parse(_frockfile.toString());
    } catch (e) {
      throw new Error('Error parsing frockfile: ' + e);
    }

    var frock = createFrockInstance(frockfile, argv);

    // run frock, create the watcher if requested
    frock.run(function () {
      if (!argv.nowatch) {
        createWatcher(frock, file);
      }

      ready(null, { argv: argv, frock: frock, frockfile: frockfile, launched: true });
    });
  }
}