'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var bole = require('bole');
var evidence = require('evidence');

var _require = require('./utils'),
    dup = _require.dup;

var log = bole('frock/register-config');
var DEFAULT_WHITELIST = ['127.0.0.1', '::1'];

module.exports = createConfigRegister;
module.exports.DEFAULT_WHITELIST = DEFAULT_WHITELIST;

function createConfigRegister() {
  var configStore = evidence();

  configStore.on('data', function () {
    return log.debug('using new configuration');
  });

  return {
    register: register,
    get: configStore.get
  };

  function register(_config) {
    if (!_config.connection) {
      _config.connection = {};
    }

    var _config$connection = _config.connection,
        whitelist = _config$connection.whitelist,
        blacklist = _config$connection.blacklist;

    // default to localhost only connections if not specified

    if (!whitelist && !blacklist) {
      log.debug('using default whitelist');

      _config.connection.whitelist = DEFAULT_WHITELIST;
    }

    configStore.write(_config);

    return dup(_config);
  }
}