'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var fs = require('fs');

var _require = require('events'),
    EventEmitter = _require.EventEmitter;

var chokidar = require('chokidar');
var bole = require('bole');

var log = bole('frock/watcher');

module.exports = createWatcher;

function createWatcher(frock, file) {
  var events = new EventEmitter();

  chokidar.watch(file, { usePolling: true, interval: 1000 }).on('change', onFrockfileChange).on('raw', function (event, path, details) {
    return log.debug(event, path, details);
  });

  return events;

  function onFrockfileChange(path) {
    fs.readFile(path, function (err, config) {
      var frockfile = void 0;

      if (err) {
        events.emit('error', err);

        return log.error('Error hot-reloading frockfile', err);
      }

      try {
        frockfile = JSON.parse(config.toString());
      } catch (e) {
        log.error('Error parsing frockfile: ' + e, e);

        return;
      }

      events.emit('change', frockfile);

      frock.reload(frockfile, function () {
        log.info('Reloaded on frockfile file change');

        events.emit('reload', frockfile);
      });
    });
  }
}